@@ -1,9 +1,11 @@
 use std::convert::TryFrom;
 use std::marker::PhantomData;
 
-// TODO With const_evaluatable_checked, FieldSize and Field could be merged by adding a size() function to Field,
-// but we'd need https://github.com/rust-lang/rust/issues/76560 first.
+/// This is an internal type that only exists because [const_evaluatable_checked](https://github.com/rust-lang/rust/issues/76560) isn't stabilized yet.
+/// TODO Once [const_evaluatable_checked](https://github.com/rust-lang/rust/issues/76560) is stabilized, we should delete this and
+/// instead add a SIZE constant to [Field].
 pub trait FieldSize {
+    /// Returns the size the data type would occupy in a layout.
     const SIZE: usize;
 }
 
@@ -18,13 +20,33 @@ pub trait Endianness {
     const KIND: EndianKind;
 }
 
-/// This is a marker type to mark layouts using big endian encoding
+/// This is a marker type to mark layouts using big endian encoding. The alternative is [LittleEndian] encoding.
+///
+/// # Example
+/// ```
+/// use binary_layout::prelude::*;
+///
+/// define_layout!(my_layout, BigEndian, {
+///   field1: i16,
+///   field2: u32,
+/// });
+/// ```
 pub struct BigEndian {}
 impl Endianness for BigEndian {
     const KIND: EndianKind = EndianKind::Big;
 }
 
-/// This is a marker type to mark layouts using little endian encoding
+/// This is a marker type to mark layouts using little endian encoding. The alternative is [BigEndian] encoding.
+///
+/// # Example
+/// ```
+/// use binary_layout::prelude::*;
+///
+/// define_layout!(my_layout, LittleEndian, {
+///   field1: i16,
+///   field2: u32,
+/// });
+/// ```
 pub struct LittleEndian {}
 impl Endianness for LittleEndian {
     const KIND: EndianKind = EndianKind::Little;
@@ -47,11 +69,9 @@ pub trait FieldMetadata {
     ///   field3: u8,
     /// });
     ///
-    /// fn main() {
-    ///     assert_eq!(0, my_layout::field1::OFFSET);
-    ///     assert_eq!(2, my_layout::field2::OFFSET);
-    ///     assert_eq!(6, my_layout::field3::OFFSET);
-    /// }
+    /// assert_eq!(0, my_layout::field1::OFFSET);
+    /// assert_eq!(2, my_layout::field2::OFFSET);
+    /// assert_eq!(6, my_layout::field3::OFFSET);
     /// ```
     const OFFSET: usize;
 }
@@ -74,11 +94,9 @@ pub trait SizedFieldMetadata {
     ///   field3: u8,
     /// });
     ///
-    /// fn main() {
-    ///     assert_eq!(2, my_layout::field1::SIZE);
-    ///     assert_eq!(4, my_layout::field2::SIZE);
-    ///     assert_eq!(1, my_layout::field3::SIZE);
-    /// }
+    /// assert_eq!(2, my_layout::field1::SIZE);
+    /// assert_eq!(4, my_layout::field2::SIZE);
+    /// assert_eq!(1, my_layout::field3::SIZE);
     /// ```
     const SIZE: usize;
 }
@@ -388,7 +406,7 @@ impl<E: Endianness, const OFFSET_: usize> Field<[u8], E, OFFSET_> {
         "},
         #[allow(dead_code)]
         pub fn data(storage: &[u8]) -> &[u8] {
-            &storage.as_ref()[Self::OFFSET..]
+            &storage[Self::OFFSET..]
         }
     }
 }
@@ -417,7 +435,7 @@ impl<E: Endianness, const OFFSET_: usize> Field<[u8], E, OFFSET_> {
         "},
         #[allow(dead_code)]
         pub fn data_mut(storage: &mut [u8]) -> &mut [u8] {
-            &mut storage.as_mut()[Self::OFFSET..]
+            &mut storage[Self::OFFSET..]
         }
     }
 }
@@ -588,7 +606,7 @@ impl<E: Endianness, const N: usize, const OFFSET_: usize> Field<[u8; N], E, OFFS
         "},
         #[allow(dead_code)]
         pub fn data(storage: &[u8]) -> &[u8; N] {
-            <&[u8; N]>::try_from(&storage.as_ref()[Self::OFFSET..(Self::OFFSET + N)]).unwrap()
+            <&[u8; N]>::try_from(&storage[Self::OFFSET..(Self::OFFSET + N)]).unwrap()
         }
     }
 }
@@ -618,7 +636,7 @@ impl<E: Endianness, const N: usize, const OFFSET_: usize> Field<[u8; N], E, OFFS
         "},
         #[allow(dead_code)]
         pub fn data_mut(storage: &mut [u8]) -> &mut [u8; N] {
-            <&mut [u8; N]>::try_from(&mut storage.as_mut()[Self::OFFSET..(Self::OFFSET + N)]).unwrap()
+            <&mut [u8; N]>::try_from(&mut storage[Self::OFFSET..(Self::OFFSET + N)]).unwrap()
         }
     }
 }
@@ -815,7 +833,7 @@ mod tests {
         );
 
         assert_eq!(10i32.pow(8), Field1::read(&storage));
-        assert_eq!(-10i32.pow(7), Field2::read(&storage));
+        assert_eq!(-(10i32.pow(7)), Field2::read(&storage));
 
         assert_eq!(4, Field::<i32, LittleEndian, 5>::SIZE);
         assert_eq!(4, Field::<i32, LittleEndian, 5>::SIZE);
@@ -841,7 +859,7 @@ mod tests {
         );
 
         assert_eq!(10i32.pow(8), Field1::read(&storage));
-        assert_eq!(-10i32.pow(7), Field2::read(&storage));
+        assert_eq!(-(10i32.pow(7)), Field2::read(&storage));
 
         assert_eq!(4, Field::<i32, BigEndian, 5>::SIZE);
         assert_eq!(4, Field::<i32, BigEndian, 5>::SIZE);
@@ -867,7 +885,7 @@ mod tests {
         );
 
         assert_eq!(10i64.pow(15), Field1::read(&storage));
-        assert_eq!(-10i64.pow(14), Field2::read(&storage));
+        assert_eq!(-(10i64.pow(14)), Field2::read(&storage));
 
         assert_eq!(8, Field::<i64, LittleEndian, 5>::SIZE);
         assert_eq!(8, Field::<i64, LittleEndian, 5>::SIZE);
@@ -893,7 +911,7 @@ mod tests {
         );
 
         assert_eq!(10i64.pow(15), Field1::read(&storage));
-        assert_eq!(-10i64.pow(14), Field2::read(&storage));
+        assert_eq!(-(10i64.pow(14)), Field2::read(&storage));
 
         assert_eq!(8, Field::<i64, BigEndian, 5>::SIZE);
         assert_eq!(8, Field::<i64, BigEndian, 5>::SIZE);
