@@ -73,7 +73,7 @@
 #[macro_export]
 macro_rules! define_layout {
     ($name: ident, $endianness: ident, {$($field_name: ident : $field_type: ty),* $(,)?}) => {
-        /// This module is autogenerated. It defines a layout using the [binary-layout](crate) crate.
+        /// This module is autogenerated. It defines a layout using the [binary_layout](crate) crate.
         #[allow(dead_code)]
         pub mod $name {
             #[allow(unused_imports)]
@@ -87,12 +87,20 @@ macro_rules! define_layout {
             pub struct View<S> {
                 storage: S,
             }
-            // TODO Finish documentation
             impl <S> View<S> {
+                /// You can create views over a storage by calling [View::new].
+                ///
+                /// `S` is the type of underlying storage. It can be
+                /// - Immutable borrowed storage: `&[u8]`
+                /// - Mutable borrowed storage: `&mut [u8]`
+                /// - Owning storage: impl `AsRef<u8>` (for example: `Vec<u8>`)
                 pub fn new(storage: S) -> Self {
                     Self {storage}
                 }
 
+                /// This destroys the view and returns the underlying storage back to you.
+                /// This is useful if you created an owning view (e.g. vased on `Vec<u8>`)
+                /// and now need the underlying `Vec<u8>` back.
                 pub fn into_storage(self) -> S {
                     self.storage
                 }
@@ -111,7 +119,7 @@ macro_rules! define_layout {
     (_impl_fields $endianness: ty, $offset_accumulator: expr, {}) => {};
     (_impl_fields $endianness: ty, $offset_accumulator: expr, {$name: ident : $type: ty $(, $name_tail: ident : $type_tail: ty)*}) => {
         doc_comment::doc_comment!{
-            concat!("Metadata and [Field](crate::Field) API accessors for the `", stringify!($name), "` field"),
+            concat!("Metadata and [Field](binary_layout::Field) API accessors for the `", stringify!($name), "` field"),
             #[allow(non_camel_case_types)]
             pub type $name = $crate::Field::<$type, $endianness, $offset_accumulator>;
         }
@@ -120,8 +128,11 @@ macro_rules! define_layout {
 
     (_impl_view_asref {}) => {};
     (_impl_view_asref {$name: ident $(, $name_tail: ident)*}) => {
-        pub fn $name(&self) -> $crate::FieldView::<&[u8], $name> {
-            $crate::FieldView::new(self.storage.as_ref())
+        doc_comment::doc_comment!{
+            concat!("Return a [FieldView](binary_layout::FieldView) with read access to the `", stringify!($name), "` field"),
+            pub fn $name(&self) -> $crate::FieldView::<&[u8], $name> {
+                $crate::FieldView::new(self.storage.as_ref())
+            }
         }
         $crate::define_layout!(_impl_view_asref {$($name_tail),*});
     };
@@ -129,8 +140,11 @@ macro_rules! define_layout {
     (_impl_view_asmut {}) => {};
     (_impl_view_asmut {$name: ident $(, $name_tail: ident)*}) => {
         paste::paste!{
-            pub fn [<$name _mut>](&mut self) -> $crate::FieldView::<&mut [u8], $name> {
-                $crate::FieldView::new(self.storage.as_mut())
+            doc_comment::doc_comment!{
+                concat!("Return a [FieldView](binary_layout::FieldView) with write access to the `", stringify!($name), "` field"),
+                pub fn [<$name _mut>](&mut self) -> $crate::FieldView::<&mut [u8], $name> {
+                    $crate::FieldView::new(self.storage.as_mut())
+                }
             }
         }
         $crate::define_layout!(_impl_view_asmut {$($name_tail),*});
@@ -139,8 +153,11 @@ macro_rules! define_layout {
     (_impl_view_into {}) => {};
     (_impl_view_into {$name: ident $(, $name_tail: ident)*}) => {
         paste::paste!{
-            pub fn [<into_ $name>](self) -> $crate::FieldView::<S, $name> {
-                $crate::FieldView::new(self.storage)
+            doc_comment::doc_comment!{
+                concat!("Destroy the [View] and return a field accessor to the `", stringify!($name), "` field owning the storage. This is mostly useful for [FieldView::extract](binary_layout::FieldView::extract)"),
+                pub fn [<into_ $name>](self) -> $crate::FieldView::<S, $name> {
+                    $crate::FieldView::new(self.storage)
+                }
             }
         }
         $crate::define_layout!(_impl_view_into {$($name_tail),*});
